# -*- coding: utf-8
import threading
from unittest import TestCase, TestResult

from django.conf import settings
from django.test import SimpleTestCase
from django.test.utils import get_runner
from optic_django_unittest.container import (
    TestCaseInteractionContainer,
    TestResultInteractionContainer,
)

from optic_django_middleware.apps import OpticDjangoAppConfig
from optic_django_middleware.manager import BasicOpticManager

local = threading.local()


class HttpInteractionManager(BasicOpticManager):
    LOCAL_TESTCASE_CONTAINER_NAME = "interaction_test_case_container"
    test_result_container: TestResultInteractionContainer = None

    @classmethod
    def get_interaction_container(cls) -> TestCaseInteractionContainer:
        return getattr(local, cls.LOCAL_TESTCASE_CONTAINER_NAME, None)

    @classmethod
    def set_interaction_container(cls, value):
        setattr(local, cls.LOCAL_TESTCASE_CONTAINER_NAME, value)

    @classmethod
    def wrap_pre_set_up(cls, set_up):
        def wrapped(self, *args, **kwargs):
            result = set_up(self, *args, **kwargs)
            if OpticDjangoAppConfig.enabled():
                cls.set_interaction_container(TestCaseInteractionContainer())
            return result

        return wrapped

    @classmethod
    def wrap_post_tear_down(cls, tear_down):
        def wrapped(self, *args, **kwargs):
            if (
                not hasattr(cls, "test_result_container")
                or not OpticDjangoAppConfig.enabled()
            ):
                return tear_down(self, *args, **kwargs)

            container = cls.get_interaction_container()
            if container is not None:
                all_interactions = cls.test_result_container
                all_interactions.add(self.id(), container)
                cls.set_interaction_container(None)
            return tear_down(self, *args, **kwargs)

        return wrapped

    @classmethod
    def patch_test_case(cls):
        SimpleTestCase._pre_setup = cls.wrap_pre_set_up(SimpleTestCase._pre_setup)

        SimpleTestCase._post_teardown = cls.wrap_post_tear_down(
            SimpleTestCase._post_teardown
        )

    @classmethod
    def wrap_setup_test_environment(cls, func):
        def wrapped(self, *args, **kwargs):
            result = func(self, *args, **kwargs)
            if not OpticDjangoAppConfig.enabled():
                return result
            cls.test_result_container = TestResultInteractionContainer()
            return result

        return wrapped

    @classmethod
    def wrap_teardown_test_environment(cls, func):
        def wrapped(self, *args, **kwargs):
            result = func(self, *args, **kwargs)
            if not OpticDjangoAppConfig.enabled():
                return result
            interactions = cls.test_result_container.get_interaction_list()
            manager = OpticDjangoAppConfig.get_manager()
            if manager:
                manager.optic.send_interactions(interactions)
            cls.test_result_container = None
            return result

        return wrapped

    @classmethod
    def patch_runner(cls):
        # FIXME: this is incompatible with --parallel and --test-runner
        # command arguments
        test_runner = get_runner(settings)

        if not hasattr(test_runner, "setup_test_environment") or not hasattr(
            test_runner, "teardown_test_environment"
        ):
            return

        test_runner.setup_test_environment = cls.wrap_setup_test_environment(
            test_runner.setup_test_environment
        )
        test_runner.teardown_test_environment = cls.wrap_teardown_test_environment(
            test_runner.teardown_test_environment
        )

    @classmethod
    def set_up_additional(cls):
        cls.patch_test_case()
        cls.patch_result()
        cls.patch_runner()

    @classmethod
    def patch_result(cls):
        TestResult.addSuccess = cls.wrap_result_handler(TestResult.addSuccess, True)
        TestResult.addFailure = cls.wrap_result_handler(TestResult.addFailure, False)
        TestResult.addError = cls.wrap_result_handler(TestResult.addError, False)

    @classmethod
    def wrap_result_handler(cls, result_handler, success):
        """
        Intercepts Result handler hooks to update test containers about the status of the test result
        """

        def wrapped(self, test: TestCase, *args, **kwargs):
            if (
                not hasattr(cls, "test_result_container")
                or cls.test_result_container is None
                or not OpticDjangoAppConfig.enabled()
            ):
                return result_handler(self, test, *args, **kwargs)
            cls.test_result_container.update_test_result_status(test.id(), success)
            result = result_handler(self, test, *args, **kwargs)
            return result

        return wrapped



from __future__ import annotations

from functools import reduce
from typing import List

from optic_django_middleware.container import BasicInteractionContainer


class TestResultInteractionContainer(object):
    """Stores all the http interactions from a Test Run, aggregated by Test Case"""

    def __init__(self):
        self.interactions_by_testcase = dict()

    def add(self, test_case_id, http_interaction_container):
        """
        Merge the http_interactions from a test case
        :param test_case_id: identifier for test case (This is usually the
            full name of the test method, including the module and class name)
        :param http_interaction_container: TestCaseInteractionContainer for this test case
        """
        existing_interaction_container = self.interactions_by_testcase.get(
            test_case_id,
            {'interaction': TestCaseInteractionContainer(), 'success': True}
        )
        existing_interaction_container['interaction'].merge(http_interaction_container)
        self.interactions_by_testcase[test_case_id] = existing_interaction_container

    def update_test_result_status(self, test_case_id, success):
        existing_interaction_container = self.interactions_by_testcase.get(
            test_case_id,
            {'interaction': TestCaseInteractionContainer(), 'success': True}
        )
        if existing_interaction_container is not None and existing_interaction_container['success'] is True:
            existing_interaction_container['success'] = success
        self.interactions_by_testcase[test_case_id] = existing_interaction_container

    def get_interaction_list(self, include_failure=False) -> List:
        interactions = self.interactions_by_testcase
        if include_failure is False:
            interactions = dict(filter(lambda interaction: interaction[1]['success'] is True, interactions.items()))
        return list(reduce((lambda a, b: a + b[1]['interaction'].http_interactions), interactions.items(), []))


class TestCaseInteractionContainer(BasicInteractionContainer):
    """Stores http interactions by API for a particular test case"""
    http_interactions: list

    def __init__(self, http_interactions: list = None):
        super().__init__()
        self.http_interactions = http_interactions or list()

    def on_interaction_captured(self, serialized_interaction):
        self.http_interactions.append(serialized_interaction)

    def merge(self, test_case_container: TestCaseInteractionContainer):
        """
        Merges the http interactions from another test case container in this object
        """
        self.http_interactions += test_case_container.http_interactions



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['optic_django_unittest']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.2.4,<4.0.0',
 'kubi-ecs-logger>=0.1.0,<0.2.0',
 'optic-django-middleware>=0.1.0,<0.2.0']

setup_kwargs = {
    'name': 'optic-django-unittest-plugin',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'Mukesh',
    'author_email': 'mmukesh95@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
